package com.feedback.demo.controllers;

import com.feedback.demo.dto.FeedBackDTO;
import com.feedback.demo.service.FeedBackService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.media.ArraySchema;
import io.swagger.v3.oas.annotations.media.Content;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.responses.ApiResponse;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.List;

@RestController
@RequestMapping("/feedback")
@Tag(name = "Feed Back")
public class FeedBackController {
    @Autowired
    private FeedBackService feedBackService;

    @RequestMapping(value = {"/show/", "/show"})
    public void redirect(HttpServletResponse response) throws IOException {
        response.sendRedirect("/feedback/show/1");
    }


    @Operation(description = "Get feed back", responses = {
            @ApiResponse(content = @Content(array = @ArraySchema(schema = @Schema(implementation = FeedBackDTO.class))), responseCode = "200")
    })
    @GetMapping("/list")
    public ResponseEntity<List<FeedBackDTO>> getFeedBack(
            @RequestParam(value = "page", required = false, defaultValue = "0") String pageIndex
    ) {
            return ResponseEntity.ok(feedBackService.getFeedback(pageIndex));
    }


    @Operation(description = "Get feed back by id", responses = {
            @ApiResponse(content = @Content(array = @ArraySchema(schema = @Schema(implementation =FeedBackDTO.class))), responseCode = "200")
    })
    @GetMapping("/show/{id}")
    public FeedBackDTO getFeedbackById(
            @PathVariable(name = "id", required = false) String id
    ) {
        return feedBackService.getFeedBackById(id);
    }
}
