package com.feedback.demo.controllers;

import com.feedback.demo.dto.login.ChangePasswordRequest;
import com.feedback.demo.dto.login.LoginRequest;
import com.feedback.demo.dto.login.LoginResponse;
import com.feedback.demo.dto.login.SignupRequest;
import com.feedback.demo.repositories.UserRepository;
import com.feedback.demo.service.JwtService;
import com.feedback.demo.service.UserService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.media.ArraySchema;
import io.swagger.v3.oas.annotations.media.Content;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.responses.ApiResponse;
import io.swagger.v3.oas.annotations.responses.ApiResponses;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.io.IOException;

@RestController
@Tag(name = "User")
public class UserController {
    @Autowired
    private UserService userService;
    @Autowired
    UserRepository userRepository;

    @Autowired
    JwtService jwtService;

    @Operation(description = "Login", responses = {
            @ApiResponse(content = @Content(
                    array = @ArraySchema(schema = @Schema(implementation = LoginRequest.class))),
                    responseCode = "200"
            )
    })
    @ApiResponses(value = {
            @ApiResponse(responseCode = "200", description = "Thành công"),
            @ApiResponse(responseCode = "404", description = "Không tìm thấy"),
            @ApiResponse(responseCode = "401", description = "Chưa xác thực"),
            @ApiResponse(responseCode = "403", description = "Truy cập bị cấm"),
    })
    @PostMapping("/auth/login")
    public LoginResponse login(@Valid @RequestBody LoginRequest loginRequest) {
        return userService.getLoginResponse(loginRequest.getUsername(), loginRequest.getPassword());
    }

    @Operation(description = "Change Password", responses = {
            @ApiResponse(content = @Content(
                    array = @ArraySchema(schema = @Schema(implementation = ChangePasswordRequest.class))),
                    responseCode = "200"
            )
    })
    @PostMapping("/auth/changepass")
    public LoginResponse changePassword(
            @Valid @RequestBody ChangePasswordRequest changePasswordRequest
    ) {
        return userService.changePassword(changePasswordRequest);
    }

    @Operation(description = "Sign up", responses = {
            @ApiResponse(content = @Content(
                    array = @ArraySchema(schema = @Schema(implementation = SignupRequest.class))),
                    responseCode = "200"
            )
    })
    @PostMapping("/auth/signup")
    public LoginResponse signup(@Valid @RequestBody SignupRequest signupRequest) throws Exception {
        System.out.println(signupRequest);
        return userService.signUp(signupRequest);
    }

    @Operation(description = "Validate token", responses = {
            @ApiResponse(content = @Content(
                    array = @ArraySchema(schema = @Schema(implementation = String.class))),
                    responseCode = "200"
            )
    })
    @GetMapping("/auth/verify/{token}")
    public LoginResponse checkToken(@PathVariable String token) throws IOException {
        return userService.checkToken(token);
    }
}
