package com.feedback.demo.entities;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
import lombok.*;
import org.springframework.data.annotation.CreatedDate;
import org.springframework.data.annotation.LastModifiedDate;
import org.springframework.data.jpa.domain.support.AuditingEntityListener;

import javax.persistence.*;
import javax.validation.constraints.NotNull;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@Entity
@Data
@AllArgsConstructor
@NoArgsConstructor
@Builder
@EntityListeners(AuditingEntityListener.class)
@Table(name = "userr")
@ToString(exclude = {"feedBacks"})
public class User {
    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    private Long id;

    @NotNull
    @Column(unique = true)
    private String username;

    @NotNull
    @Column(unique = true)
    private String password;

    @Column(unique = true,length = 300)
    private String email;

    @Column(length = 20)
    private String phone;

    @Column(length = 200)
    private String domain;

    @Column(length = 2000)
    private String description;

    @Column(length = 200)
    private String appKey;

    @Column(length =200)
    private String appSecret;

    @CreatedDate
    private Date createDate;
    @LastModifiedDate
    private Date updateDate;

    @OneToMany(mappedBy ="user",fetch = FetchType.LAZY,cascade = CascadeType.ALL)
    private List<FeedBack> feedBacks = new ArrayList<>();

    @OneToMany(mappedBy = "user",cascade = CascadeType.ALL,fetch = FetchType.LAZY)
    private List<FeedBackOptions> feedBackOptions;

    @OneToOne(mappedBy = "user",cascade = CascadeType.ALL,fetch = FetchType.LAZY,optional = false)
    private FeedBackSettings feedBackSettings;
}
