package com.feedback.demo.exceptionhandle;

import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.authentication.BadCredentialsException;
import org.springframework.security.authentication.InternalAuthenticationServiceException;
import org.springframework.validation.FieldError;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.annotation.ControllerAdvice;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.context.request.WebRequest;
import org.springframework.web.servlet.mvc.method.annotation.ResponseEntityExceptionHandler;

import java.util.HashMap;
import java.util.List;
import java.util.stream.Collectors;

@ControllerAdvice
@RestController
public class CustomResponseException extends ResponseEntityExceptionHandler {

    @ExceptionHandler(Exception.class)
    public final ResponseEntity<Object> handlerAllException(Exception ex, WebRequest webRequest) {
        ExceptionResponse<String> exceptionResponse = new ExceptionResponse<>(
                HttpStatus.INTERNAL_SERVER_ERROR.value(), ex.getMessage());
        return new ResponseEntity<>(exceptionResponse, HttpStatus.INTERNAL_SERVER_ERROR);
    }

    @Override
    protected ResponseEntity<Object> handleMethodArgumentNotValid(
            MethodArgumentNotValidException ex, HttpHeaders headers, HttpStatus status, WebRequest request) {
        List<HashMap<String, String>> message = convertToMessage(ex.getBindingResult().getFieldErrors());
        ExceptionResponse<List<HashMap<String, String>>> exceptionResponse = new ExceptionResponse<>(HttpStatus.BAD_REQUEST.value(), message);
        return new ResponseEntity<>(exceptionResponse, HttpStatus.BAD_REQUEST);
    }

    @ExceptionHandler(UsernameIsAlreadyTakenException.class)
    public final ResponseEntity<Object> handlerUsernameIsAlreadyTakenException(
            UsernameIsAlreadyTakenException ex, WebRequest webRequest) {
        ExceptionResponse<String> exceptionResponse = new ExceptionResponse<>(HttpStatus.CONFLICT.value(), ex.getMessage());
        return new ResponseEntity<>(exceptionResponse, HttpStatus.CONFLICT);
    }

    @ExceptionHandler(BadCredentialsException.class)
    public final ResponseEntity<Object> handlerUnAuthorizedException(Exception ex, WebRequest webRequest) {
        ExceptionResponse<String> exceptionResponse = new ExceptionResponse<>(HttpStatus.UNAUTHORIZED.value(),
                "UnAuthorized , Username or Password incorrectly !");
        return new ResponseEntity<>(exceptionResponse, HttpStatus.UNAUTHORIZED);
    }

    @ExceptionHandler(PasswordNotMatchException.class)
    public final ResponseEntity<Object> handlePassWordNotMatchException(PasswordNotMatchException ex,
                                                                        WebRequest webRequest) {
        ExceptionResponse<String> exceptionResponse = new ExceptionResponse<>(HttpStatus.BAD_REQUEST.value(), ex.getMessage());
        return new ResponseEntity<>(exceptionResponse, HttpStatus.BAD_REQUEST);
    }

    @ExceptionHandler(ApiMissingException.class)
    public final ResponseEntity<Object> handleApiMissingException(ApiMissingException ex,
                                                                  WebRequest webRequest) {
        ExceptionResponse<String> exceptionResponse = new ExceptionResponse<>(HttpStatus.UNAUTHORIZED.value(), ex.getMessage());
        return new ResponseEntity<>(exceptionResponse, HttpStatus.UNAUTHORIZED);
    }

    @ExceptionHandler(InternalAuthenticationServiceException.class)
    public final ResponseEntity<Object> handleInternalAuthenticationServiceException(
            InternalAuthenticationServiceException ex, WebRequest webRequest) {
        ExceptionResponse<String> exceptionResponse = new ExceptionResponse<>(HttpStatus.UNAUTHORIZED.value(),
                "UnAuthorized , Username or Password incorrectly");
        return new ResponseEntity<>(exceptionResponse, HttpStatus.UNAUTHORIZED);
    }

    public List<HashMap<String, String>> convertToMessage(List<FieldError> fieldErrors) {
        return fieldErrors.stream().map(fieldError -> {
            HashMap<String, String> temp = new HashMap<>();
            temp.put("key", fieldError.getField());
            temp.put("value", fieldError.getDefaultMessage());
            return temp;
        }).collect(Collectors.toList());
    }
}
