package com.feedback.demo.service;

import com.feedback.demo.dto.FeedBackDTO;
import com.feedback.demo.entities.FeedBack;
import com.feedback.demo.exceptionhandle.FeedBackNotFoundException;
import com.feedback.demo.exceptionhandle.PageNotFoundException;
import com.feedback.demo.mappers.FeedBackMapper;
import com.feedback.demo.repositories.FeedBackRepository;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Sort;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.stream.Collectors;

@Service
@Transactional(rollbackFor = Exception.class)
@Slf4j
public class FeedBackService {
    @Autowired
    private FeedBackRepository feedBackRepository;

    @Value("${page.size:10}")
    private Integer pageSize;

    @Value("${sort.default:createDate}")
    private String sortBy;

    public List<FeedBackDTO> getFeedback(String pageIndex) {
        try {
            int page = Integer.parseInt(pageIndex);
            Page<FeedBack> feedBackPage = feedBackRepository
                    .findAll(PageRequest.of(page, pageSize, Sort.by(sortBy)));
            if (page > feedBackPage.getTotalPages()) {
                throw new PageNotFoundException();
            }
            return feedBackPage.stream()
                    .map(FeedBackMapper::toDTO)
                    .collect(Collectors.toList());
        } catch (NumberFormatException e) {
            log.error(e.getMessage());
            throw new FeedBackNotFoundException();
        }
    }

    public FeedBackDTO getFeedBackById(String id) {
        try {
            Long feedBackId = Long.parseLong(id);
            return feedBackRepository
                    .findById(feedBackId)
                    .map(FeedBackMapper::toDTO)
                    .orElseThrow(FeedBackNotFoundException::new);
        } catch (NumberFormatException e) {
            log.error(e.getMessage());
            throw new FeedBackNotFoundException();
        }
    }
}
