package com.feedback.demo.service;

import com.feedback.demo.dto.FeedBackOptionsDTO;
import com.feedback.demo.dto.FeedBackSettingsRequest;
import com.feedback.demo.entities.FeedBackOptions;
import com.feedback.demo.entities.FeedBackSettings;
import com.feedback.demo.entities.User;
import com.feedback.demo.mappers.FeedBackSettingsMapper;
import com.feedback.demo.repositories.FeedBackOptionsRepository;
import com.feedback.demo.repositories.FeedBackSettingsRepository;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Comparator;
import java.util.List;

@Service
@Transactional(rollbackFor = Exception.class)
@Slf4j
public class FeedBackSettingsService {
    @Autowired
    private FeedBackSettingsRepository feedBackSettingsRepository;

    @Autowired
    private FeedBackOptionsRepository feedBackOptionsRepository;
    @Autowired
    private UserService userService;

    public FeedBackSettingsRequest updateSettings(FeedBackSettingsRequest fsRequest) {
        User user = userService.getCurrentUser();
        FeedBackSettings settings = feedBackSettingsRepository.findByUserId(user.getId());
        List<FeedBackOptions> options = feedBackOptionsRepository.findByUserId(user.getId());
        if (settings == null) {
            throw new RuntimeException("setting not available ! ");
        }
        if (options == null) {
            throw new RuntimeException("option not available ! ");
        }
        updateSettings(settings, fsRequest);
        updateOptions(options, fsRequest.getOptionSettings());
        try {
            feedBackSettingsRepository.save(settings);
            feedBackOptionsRepository.saveAll(options);

        } catch (Exception e) {
            log.error(e.getMessage());
            throw new RuntimeException("Setting error ! ");
        }
        return fsRequest;
    }

    public FeedBackSettingsRequest getFeedbackSettings() {
        User user = userService.getCurrentUser();
        FeedBackSettings settings = feedBackSettingsRepository.findByUserId(user.getId());
        List<FeedBackOptions> options = feedBackOptionsRepository.findByUserId(user.getId());
        if (settings == null) {
            throw new RuntimeException("setting not available ! ");
        }
        if (options == null) {
            throw new RuntimeException("option not available ! ");
        }
        options.sort(Comparator.comparingInt(FeedBackOptions::getOptionOrder));
        return FeedBackSettingsMapper.toFBSettingsRequest(settings,options);
    }

    private void updateSettings(FeedBackSettings fs, FeedBackSettingsRequest fsRequest) {
        fs.setPosition(fsRequest.getPosition());
        fs.setIconURL(fsRequest.getIconURL());
        fs.setFeedbackTitle(fsRequest.getFeedbackTitle());
        fs.setTitleFullName(fsRequest.getTitleFullName());
        fs.setTitleEmail(fsRequest.getTitleEmail());
        fs.setTitlePhoneNumber(fsRequest.getTitlePhoneNumber());
        fs.setTitleOptions(fsRequest.getTitleOptions());
        fs.setTitleDescription(fsRequest.getTitleDescription());
    }

    private void updateOptions(List<FeedBackOptions> foList, List<FeedBackOptionsDTO> foRequest) {
        foList.forEach(fo -> {
            foRequest.stream()
                    .filter(i -> i.getId().equals(fo.getId()))
                    .forEach(i -> {
                        fo.setOptionOrder(i.getOptionOrder());
                        fo.setOptionValue(i.getOptionValue());
                    });
        });
    }
}
