package com.feedback.demo.service;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.feedback.demo.config.CustomUserDetail;
import com.feedback.demo.dto.Payload;
import com.feedback.demo.entities.User;
import com.feedback.demo.repositories.UserRepository;
import io.jsonwebtoken.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.util.Base64;

@Service
public class JwtService {
    @Autowired
    UserRepository userRepository;

    public String generateToken(CustomUserDetail customUserDetail) {
        return Jwts.builder().setSubject(Long.toString(customUserDetail.getUser().getId()))
                .signWith(SignatureAlgorithm.HS256, customUserDetail.getUser().getPassword()).compact();
    }

    public User getUserFromToken(String token) throws IOException {
        String[] part = token.split("\\.");
        //lay cai payload ra
        byte[] decodedBytes = Base64.getUrlDecoder().decode(part[1]);
        String payload = new String(decodedBytes);
        ObjectMapper mapper = new ObjectMapper();
        Payload result = mapper.readValue(payload, Payload.class);
        int id = result.getSub();
        User user = userRepository.findById((long) id).orElse(null);
        if (user != null) {
            Claims claims = Jwts.parser().setSigningKey(user.getPassword()).parseClaimsJws(token).getBody();
            return (Integer.parseInt(claims.getSubject()) == id && validateToken(token, user.getPassword())) ? user : null;
        }
        return null;
    }

    public boolean validateToken(String authToken, String secret) {
        try {
            Jwts.parser().setSigningKey(secret).parseClaimsJws(authToken);
            return true;
        } catch (ExpiredJwtException | UnsupportedJwtException
                | MalformedJwtException | SignatureException | IllegalArgumentException e) {
            e.printStackTrace();
        }
        return false;
    }
}
